import numpy as np, matplotlib.pyplot as plt
from scipy.optimize import curve_fit

class Donnees():
    def __init__(self, chemin_fichier=""):
        if len(chemin_fichier)>0:
            self.t, self.y = np.loadtxt(chemin_fichier)
    
    def plot(self, ajustement=True, **kwargs):
        fig, ax = plt.subplots(1, 1)
        ax.plot(self.t, self.y, **kwargs)
        if ajustement:
            a, b, sigma_a, sigma_b = self.ajuster()
            t_plot = np.linspace(self.t[0], self.t[-1])
            y_plot = modele(t_plot, a, b)
            ax.plot(t_plot, y_plot,
                    label="Ajustement (a={:.2e}±{:.2e}, b={:.2e}±{:.2e})".format(a, sigma_a, b, sigma_b))
        ax.set_xlabel("t [s]")
        ax.set_ylabel("y")
        ax.grid(True)
        ax.legend(loc="best")
    
    def filtrer(self, lims):
        filtre = (self.t >= lims["t"][0]) & (self.t <= lims["t"][1]) & (self.y >= lims["y"][0]) & (self.y <= lims["y"][1])
        nouv_donnees = self.__class__()
        nouv_donnees.t = self.t[filtre]
        nouv_donnees.y = self.y[filtre]
        return nouv_donnees

    def moyenne(self, afficher=False):
        m, e = np.mean(self.y), np.std(self.y)
        if afficher: print("Moyenne : {:.2f} ± {:.2f}".format(m, e))
        return m, e
    
    def ajuster(self):
        t, y = self.t, self.y
        a_guess = (y[-1] - y[0]) / (t[-1] - t[0])
        b_guess = self.moyenne(afficher=False)[0]
        popt, pcov = curve_fit(modele, t, y, p0=(a_guess, b_guess))
        return *popt, np.sqrt(pcov[0,0]), np.sqrt(pcov[1,1])